@@ -20,7 +20,9 @@ import errno
 import hashlib
 import logging
 import os
+import re
 import shutil
+import sys
 import threading
 import weakref
 
@@ -205,6 +207,7 @@ class FileDriver(coordination._RunWatchersMixin,
     """
 
     CHARACTERISTICS = (
+        coordination.Characteristics.NON_TIMEOUT_BASED,
         coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
         coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
     )
@@ -228,7 +231,7 @@ class FileDriver(coordination._RunWatchersMixin,
         """Initialize the file driver."""
         super(FileDriver, self).__init__()
         self._member_id = member_id
-        self._dir = parsed_url.path
+        self._dir = self._normalize_path(parsed_url.path)
         self._executor = utils.ProxyExecutor.build("File", options)
         self._group_dir = os.path.join(self._dir, 'groups')
         self._driver_lock_path = os.path.join(self._dir, '.driver_lock')
@@ -240,6 +243,19 @@ class FileDriver(coordination._RunWatchersMixin,
         self._joined_groups = set()
         self._safe_member_id = self._make_filesystem_safe(member_id)
 
+    @staticmethod
+    def _normalize_path(path):
+        if sys.platform == 'win32':
+            # Replace slashes with backslashes and make sure we don't
+            # have any at the beginning of paths that include drive letters.
+            #
+            # Expected url format:
+            # file:////share_address/share_name
+            # file:///C:/path
+            return re.sub(r'\\(?=\w:\\)', '',
+                          os.path.normpath(path))
+        return path
+
     @classmethod
     def _get_raw_lock(cls, path, member_id):
         lock_barrier = cls._barriers.setdefault(path, _Barrier())
@@ -260,7 +276,10 @@ class FileDriver(coordination._RunWatchersMixin,
 
     def _start(self):
         for a_dir in self._reserved_dirs:
-            utils.ensure_tree(a_dir)
+            try:
+                utils.ensure_tree(a_dir)
+            except OSError as e:
+                raise coordination.ToozConnectionError(e)
         self._executor.start()
 
     def _stop(self):
@@ -499,7 +518,7 @@ class FileFutureResult(coordination.CoordAsyncResult):
     def __init__(self, fut):
         self._fut = fut
 
-    def get(self, timeout=10):
+    def get(self, timeout=None):
         try:
             # Late translate the common failures since the file driver
             # may throw things that we can not catch in the callbacks where
